@@ -1,5 +1,5 @@
 /*
- * $Id: gfarm_dirtree.h 8521 2013-08-15 10:56:21Z takuya-i $
+ * $Id: gfarm_dirtree.h 9753 2015-06-24 00:13:12Z takuya-i $
  */
 
 typedef struct gfarm_dirtree gfarm_dirtree_t;
@@ -24,8 +24,8 @@ typedef struct gfarm_dirtree_entry {
 	unsigned char dst_exist;
 } gfarm_dirtree_entry_t;
 
-gfarm_error_t gfarm_dirtree_init_fork(gfarm_dirtree_t **, const char *,
-	const char *, int, int, int);
+gfarm_error_t gfarm_dirtree_init_fork(gfarm_dirtree_t **, GFURL, GFURL,
+	int, int, int);
 gfarm_error_t gfarm_dirtree_open(gfarm_dirtree_t *);
 
 gfarm_error_t gfarm_dirtree_checknext(gfarm_dirtree_t *,
