@@ -1,5 +1,5 @@
 /*
- * $Id: config.c 9637 2015-04-16 03:36:02Z n-soda $
+ * $Id: config.c 9762 2015-06-24 09:59:35Z takuya-i $
  */
 
 #include <pthread.h>
@@ -932,6 +932,9 @@ int gfarm_iostat_max_client = GFARM_CONFIG_MISC_DEFAULT;
 #define GFARM_GFSD_CONNECTION_CACHE_DEFAULT 16 /* 16 free connections */
 #define GFARM_GFMD_CONNECTION_CACHE_DEFAULT  8 /*  8 free connections */
 #define GFARM_METADB_MAX_DESCRIPTORS_DEFAULT	(2*65536)
+#define GFARM_METADB_REPLICA_REMOVER_BY_HOST_SLEEP_TIME_DEFAULT	20000000
+							/* nanosec. */
+#define GFARM_METADB_REPLICA_REMOVER_BY_HOST_INODE_STEP_DEFAULT	1024
 #define GFARM_CLIENT_DIGEST_CHECK_DEFAULT	0
 #define GFARM_CLIENT_FILE_BUFSIZE_DEFAULT	(1024 * 1024)
 #define GFARM_CLIENT_PARALLEL_COPY_DEFAULT	4
@@ -948,6 +951,8 @@ int gfarm_iostat_max_client = GFARM_CONFIG_MISC_DEFAULT;
 #define GFARM_FILE_TRACE_DEFAULT 0 /* disable */
 #define GFARM_FATAL_ACTION_DEFAULT GFLOG_FATAL_ACTION_ABORT_BACKTRACE
 #define GFARM_REPLICA_CHECK_DEFAULT 1 /* enable */
+#define GFARM_REPLICA_CHECK_REMOVE_DEFAULT 1 /* enable */
+#define GFARM_REPLICA_CHECK_REDUCED_LOG_DEFAULT 1 /* enable */
 #define GFARM_REPLICA_CHECK_HOST_DOWN_THRESH_DEFAULT 10800 /* 3 hours */
 #define GFARM_REPLICA_CHECK_SLEEP_TIME_DEFAULT 100000 /* nanosec. */
 #define GFARM_REPLICA_CHECK_MINIMUM_INTERVAL_DEFAULT 10 /* 10 sec. */
@@ -962,6 +967,10 @@ int gfarm_metadb_thread_pool_size = GFARM_CONFIG_MISC_DEFAULT;
 int gfarm_metadb_job_queue_length = GFARM_CONFIG_MISC_DEFAULT;
 int gfarm_metadb_heartbeat_interval = GFARM_CONFIG_MISC_DEFAULT;
 int gfarm_metadb_dbq_size = GFARM_CONFIG_MISC_DEFAULT;
+int gfarm_metadb_replica_remover_by_host_sleep_time =
+	GFARM_CONFIG_MISC_DEFAULT;
+int gfarm_metadb_replica_remover_by_host_inode_step =
+	GFARM_CONFIG_MISC_DEFAULT;
 static int metadb_replication_enabled = GFARM_CONFIG_MISC_DEFAULT;
 static char *journal_dir = NULL;
 static int journal_max_size = GFARM_CONFIG_MISC_DEFAULT;
@@ -972,6 +981,8 @@ static int metadb_server_slave_replication_timeout = GFARM_CONFIG_MISC_DEFAULT;
 static int metadb_server_slave_max_size = GFARM_CONFIG_MISC_DEFAULT;
 static int metadb_server_force_slave = GFARM_CONFIG_MISC_DEFAULT;
 int gfarm_replica_check = GFARM_CONFIG_MISC_DEFAULT;
+int gfarm_replica_check_remove = GFARM_CONFIG_MISC_DEFAULT;
+int gfarm_replica_check_reduced_log = GFARM_CONFIG_MISC_DEFAULT;
 int gfarm_replica_check_host_down_thresh = GFARM_CONFIG_MISC_DEFAULT;
 int gfarm_replica_check_sleep_time = GFARM_CONFIG_MISC_DEFAULT;
 int gfarm_replica_check_minimum_interval = GFARM_CONFIG_MISC_DEFAULT;
@@ -2989,6 +3000,14 @@ parse_one_line(char *s, char *p, char **op)
 		e = parse_set_misc_int(p, &gfarm_metadb_heartbeat_interval);
 	} else if (strcmp(s, o = "metadb_server_dbq_size") == 0) {
 		e = parse_set_misc_int(p, &gfarm_metadb_dbq_size);
+	} else if (strcmp(s, o = "metadb_replica_remover_by_host_sleep_time")
+	     == 0) {
+		e = parse_set_misc_int(p,
+		    &gfarm_metadb_replica_remover_by_host_sleep_time);
+	} else if (strcmp(s, o = "metadb_replica_remover_by_host_inode_step")
+	    == 0) {
+		e = parse_set_misc_int(p,
+		    &gfarm_metadb_replica_remover_by_host_inode_step);
 	} else if (strcmp(s, o = "record_atime") == 0) {
 		int record_atime;
 
@@ -3044,6 +3063,10 @@ parse_one_line(char *s, char *p, char **op)
 
 	} else if (strcmp(s, o = "replica_check") == 0) {
 		e = parse_set_misc_enabled(p, &gfarm_replica_check);
+	} else if (strcmp(s, o = "replica_check_remove") == 0) {
+		e = parse_set_misc_enabled(p, &gfarm_replica_check_remove);
+	} else if (strcmp(s, o = "replica_check_reduced_log") == 0) {
+		e = parse_set_misc_enabled(p, &gfarm_replica_check_reduced_log);
 	} else if (strcmp(s, o = "replica_check_host_down_thresh") == 0) {
 		e = parse_set_misc_int(
 		    p, &gfarm_replica_check_host_down_thresh);
@@ -3304,6 +3327,14 @@ gfarm_config_set_default_misc(void)
 		    GFARM_METADB_HEARTBEAT_INTERVAL_DEFAULT;
 	if (gfarm_metadb_dbq_size == GFARM_CONFIG_MISC_DEFAULT)
 		gfarm_metadb_dbq_size = GFARM_METADB_DBQ_SIZE_DEFAULT;
+	if (gfarm_metadb_replica_remover_by_host_sleep_time
+	    == GFARM_CONFIG_MISC_DEFAULT)
+		gfarm_metadb_replica_remover_by_host_sleep_time =
+		    GFARM_METADB_REPLICA_REMOVER_BY_HOST_SLEEP_TIME_DEFAULT;
+	if (gfarm_metadb_replica_remover_by_host_inode_step
+	    == GFARM_CONFIG_MISC_DEFAULT)
+		gfarm_metadb_replica_remover_by_host_inode_step =
+		    GFARM_METADB_REPLICA_REMOVER_BY_HOST_INODE_STEP_DEFAULT;
 	if (gfarm_atime_type == GFARM_ATIME_DEFAULT)
 		(void)gfarm_atime_type_set(GFARM_ATIME_RELATIVE);
 	if (gfarm_ctxp->client_digest_check == GFARM_CONFIG_MISC_DEFAULT)
@@ -3348,6 +3379,12 @@ gfarm_config_set_default_misc(void)
 		gflog_set_fatal_action(GFARM_FATAL_ACTION_DEFAULT);
 	if (gfarm_replica_check == GFARM_CONFIG_MISC_DEFAULT)
 		gfarm_replica_check = GFARM_REPLICA_CHECK_DEFAULT;
+	if (gfarm_replica_check_remove == GFARM_CONFIG_MISC_DEFAULT)
+		gfarm_replica_check_remove =
+		    GFARM_REPLICA_CHECK_REMOVE_DEFAULT;
+	if (gfarm_replica_check_reduced_log == GFARM_CONFIG_MISC_DEFAULT)
+		gfarm_replica_check_reduced_log =
+		    GFARM_REPLICA_CHECK_REDUCED_LOG_DEFAULT;
 	if (gfarm_replica_check_host_down_thresh == GFARM_CONFIG_MISC_DEFAULT)
 		gfarm_replica_check_host_down_thresh =
 		    GFARM_REPLICA_CHECK_HOST_DOWN_THRESH_DEFAULT;
