@@ -1,5 +1,5 @@
 /*
- * $Id: gfmd.c 9657 2015-04-23 08:02:01Z n-soda $
+ * $Id: gfmd.c 9779 2015-06-30 06:31:35Z takuya-i $
  */
 
 #include <gfarm/gfarm_config.h>
@@ -58,6 +58,7 @@
 #include "group.h"
 #include "peer.h"
 #include "inode.h"
+#include "file_copy.h"
 #include "dead_file_copy.h"
 #include "process.h"
 #include "fs.h"
@@ -1367,12 +1368,7 @@ sigs_handler(void *p)
 		}
 		switch (sig) {
 		case SIGHUP:
-#ifdef HAVE_GSI
-			/* initialize the GSI environment */
-			gflog_info(GFARM_MSG_1002735,
-			    "initialize the GSI environment");
-			gfarm_gsi_server_finalize();
-#endif
+			/* nothing to do */
 			continue;
 
 		case SIGUSR1:
@@ -1818,7 +1814,7 @@ main(int argc, char **argv)
 	/* master */
 
 	failover_notify();
-	replica_check_start();
+	replica_check_init();
 	accepting_loop(sock);
 
 	/*NOTREACHED*/
