@@ -1,6 +1,5 @@
 void inode_init(void);
 void dir_entry_init(void);
-void file_copy_init(void);
 void symlink_init(void);
 void xattr_init(void);
 
@@ -16,6 +15,7 @@ struct file_copy;
 struct dead_file_copy;
 struct gfs_stat;
 struct inode_trace_log_info;
+struct replica_spec;
 
 void inode_for_each_file_copies(
 	struct inode *,
@@ -45,10 +45,8 @@ void inode_set_group_by_name_in_cache(struct inode *, const char *);
 int inode_has_no_replica(struct inode *);
 gfarm_int64_t inode_get_ncopy(struct inode *);
 gfarm_int64_t inode_get_ncopy_with_dead_host(struct inode *);
-gfarm_error_t inode_count_ncopy_with_grace(
-	struct file_copy *, int, gfarm_time_t, int, struct host **, int *);
-gfarm_error_t inode_get_ncopy_with_grace(
-	struct inode *, int, gfarm_time_t, int, struct host **, int *);
+gfarm_error_t inode_count_replicas(
+	struct inode *, int, int, gfarm_time_t, int, struct host **, int *);
 
 gfarm_mode_t inode_get_mode(struct inode *);
 gfarm_error_t inode_set_mode(struct inode *, gfarm_mode_t);
@@ -129,10 +127,12 @@ void inode_remove_replica_completed(gfarm_ino_t, gfarm_int64_t, struct host *);
 gfarm_error_t inode_remove_replica_metadata(struct inode *, struct host *,
 	gfarm_int64_t);
 gfarm_error_t inode_remove_replica_protected(struct inode *, struct host *,
-	struct file_opening *);
+	struct replica_spec *);
+gfarm_error_t inode_remove_replica_orphan(struct inode *, struct host *);
 void inode_remove_replica_incomplete(struct inode *, struct host *,
 	gfarm_int64_t);
 gfarm_error_t inode_remove_replica_in_cache(struct inode *, struct host *);
+void inode_remove_replica_in_cache_for_invalid_host(gfarm_ino_t);
 int inode_is_updated(struct inode *, struct gfarm_timespec *);
 gfarm_error_t dir_entry_add(gfarm_ino_t, char *, int, gfarm_ino_t);
 
